package org.beangle.config.loader;

import java.io.InputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.beangle.config.SystemConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PropertyBasedConfigLoader implements SystemConfigLoader {

	protected static final Logger logger = LoggerFactory.getLogger(PropertyBasedConfigLoader.class);

	public void load(SystemConfig config) {
		try {
			URL url = PropertyBasedConfigLoader.class.getResource("/system-default.properties");
			if (null != url) {
				populateConfigItems(config, url);
			}
			Enumeration<URL> em = PropertyBasedConfigLoader.class.getClassLoader().getResources(
					"META-INF/system.properties");
			while (em.hasMoreElements()) {
				populateConfigItems(config, em.nextElement());
			}
			url = SystemConfigLoader.class.getResource("/system.properties");
			if (null != url) {
				populateConfigItems(config, url);
			}
			logger.info(config.toString());
		} catch (Exception e) {
			logger.error("Exception", e);
			throw new RuntimeException(e);
		}
	}

	private void populateConfigItems(SystemConfig config, URL url) {
		logger.debug("loading {}", url);
		try {
			Properties props = new Properties();
			InputStream is = url.openStream();
			props.load(is);
			is.close();
			Map<String, String> reserved = getReservedNames();
			for (Iterator iterator = props.keySet().iterator(); iterator.hasNext();) {
				String name = (String) iterator.next();
				String newName = name;
				if (reserved.containsKey(name)) {
					newName = reserved.get(name);
				}
				if (PropertyUtils.isWriteable(config, newName)) {
					BeanUtils.copyProperty(config, newName, props.getProperty(name));
				} else {
					config.setConfigItemValue(newName, props.getProperty(name));
				}
			}
		} catch (Exception e) {
			logger.error("populate config error", e);
		}
	}

	private Map<String, String> getReservedNames() {
		Map<String, String> names = new HashMap();
		names.put("system.name", "systemName");
		names.put("system.vendor", "vendor");
		return names;
	}
}
